﻿// Copyright (C) Microsoft Corporation. All Rights Reserved.
// This code released under the terms of the Microsoft Public License
// (Ms-PL, http://opensource.org/licenses/ms-pl.html).

using System;
using System.Diagnostics;
using System.Linq.Expressions;

namespace Delay
{
    /// <summary>
    /// Provides methods to help debug your code (similar to System.Diagnostics.Debug).
    /// </summary>
    internal static class DebugEx
    {
        /// <summary>
        /// Checks for a condition; if the condition is false, outputs the condition and displays a message box that shows the call stack.
        /// </summary>
        /// <param name="conditionFunc">A delegate for the conditional expression to evaluate. If the condition is true, the condition is not sent and the message box is not displayed.</param>
        [Conditional("DEBUG")]
        public static void Assert(Expression<Func<bool>> conditionFunc)
        {
            var condition = conditionFunc.Compile().Invoke();
            if (!condition)
            {
                var message = new SimplifyingExpressionVisitor().Visit(conditionFunc.Body).ToString();
                Debug.Assert(false, message);
            }
        }

        /// <summary>
        /// Specialization of ExpressionVisitor that simplifies expressions for display.
        /// </summary>
        private class SimplifyingExpressionVisitor : ExpressionVisitor
        {
            /// <summary>
            /// Visits the children of the MemberExpression.
            /// </summary>
            /// <param name="node">The expression to visit.</param>
            /// <returns>The modified expression.</returns>
            protected override Expression VisitMember(MemberExpression node)
            {
                if (node == null)
                {
                    throw new ArgumentNullException("node");
                }

                // If Expression looks like a compiler-generated class (i.e., for a closure),
                // substitute a simple variable with the corresponding member name.
                if (node.Expression.Type.Name.StartsWith("<>", StringComparison.Ordinal))
                {
                    return Expression.Variable(node.Type, node.Member.Name);
                }
                else
                {
                    return base.VisitMember(node);
                }
            }
        }
    }
}
